import { __awaiter } from "tslib";
import { jsx as _jsx, jsxs as _jsxs } from "@emotion/react/jsx-runtime";
import { Box, Text, Button, Grid, GridItem, Image, Input, Menu, MenuButton, MenuItem, MenuList, Radio, Stack, useRadio, useRadioGroup, } from "@chakra-ui/react";
import { ChevronDown } from "@feedloop/icon";
import React from "react";
import { defineComponent, EditorContext, useSourceHooks, useEvaluate, ListItemVariables, RuntimeContext, useRuntimeInput, } from "@qorebase/app-component";
import { fallbackImg } from "@qorebase/app-runtime/src/utils/fallbackImage";
import InputComponentLabel from "@qorebase/app-runtime/src/components/InputComponentLabel";
import useDisableCondition from "@qorebase/app-runtime/src/hooks/useDisableCondition";
const Option = (props) => {
    const value = useEvaluate(props.properties.itemValue ||
        String(props.item.id) ||
        String(props.source.itemId));
    const label = useEvaluate(props.properties.itemLabel);
    if (String(props.dataValue) === String(value))
        props.setSelected(label);
    return (_jsx(MenuItem, Object.assign({ display: !!props.search &&
            !label.toLocaleLowerCase().includes(props.search.toLocaleLowerCase())
            ? "none"
            : undefined, onClick: () => props.onChange(value), backgroundColor: String(props.dataValue) === String(value) && "gray.100" }, { children: label })));
};
const RadioItem = React.memo(function RadioItem(props) {
    const value = useEvaluate(props.properties.itemValue ||
        String(props.item.id) ||
        String(props.source.itemId));
    const { getInputProps, htmlProps, state, getCheckboxProps, getLabelProps } = useRadio(props.getRadioProps({ value }));
    const image = useEvaluate(props.properties.itemImage);
    const label = useEvaluate(props.properties.itemLabel);
    return (_jsxs(GridItem, Object.assign({ bg: "surface.900", rounded: "md", border: "1px", borderColor: state.isChecked ? props.app.theme.options.primaryColor : "gray.200", style: {
            border: "1px solid rgb(226, 232, 240)",
            pointerEvents: props.isDisabled ? "none" : "auto",
            background: props.isDisabled && "#E7E7E7",
        }, _hover: !state.isChecked && {
            bg: "gray.200",
            borderColor: "gray.300",
        } }, { children: [props.properties.style === "text_choice" && (_jsx(Radio, Object.assign({ w: "full", p: "1em", color: props.properties.style === "text_choice" && "gray.300", colorScheme: props.properties.style === "text_choice"
                    ? `var(--chakra-colors-qore-primary);`
                    : undefined }, props.getRadioProps({ value }), { children: _jsx(Text, Object.assign({ color: props.isDisabled && "#A2A2A2" }, { children: label })) }))), props.properties.style === "image_choice" && (_jsxs(Box, Object.assign({ as: "label", cursor: "pointer" }, htmlProps, { children: [_jsx("input", Object.assign({}, getInputProps({}), { hidden: true })), _jsxs(Stack, Object.assign({ direction: props.properties.layout === "horizontal_component"
                            ? "row"
                            : "column", alignItems: "center", justifyContent: props.properties.layout === "vertical_component" && "center", spacing: "1em", w: "full", p: "1em", _hover: !state.isChecked &&
                            state.isHovered && { border: "1px solid gray.300" }, border: "1px solid transparent", borderColor: state.isChecked && `var(--chakra-colors-qore-primary);`, borderRadius: "md" }, getCheckboxProps(), { children: [props.properties.style === "image_choice" && (_jsx(Image, Object.assign({ src: image, alt: label, fallbackSrc: fallbackImg, w: "64px", h: "64px", borderRadius: "md", style: {
                                    border: "1px solid rgb(226, 232, 240)",
                                    filter: props.isDisabled ? "grayscale(1)" : "grayscale(0)",
                                    WebkitFilter: props.isDisabled
                                        ? "grayscale(1)"
                                        : "grayscale(0)",
                                    opacity: props.isDisabled ? "0.5" : "1",
                                } }, getLabelProps()))), _jsx(Text, Object.assign({ display: props.properties.hideLabel && "none", color: props.isDisabled && "#A2A2A2" }, { children: label }))] }))] })))] })));
});
const CustomRadioGroup = React.memo(function CustomRadioGroup(props) {
    var _a;
    const { getRootProps, getRadioProps } = useRadioGroup({
        value: props.variableData.value,
        onChange: props.variableData.onChange,
    });
    return (_jsx(Grid, Object.assign({ templateColumns: props.isVerticalAlign
            ? "repeat(1,1fr)"
            : {
                base: "repeat(1,1fr)",
                md: "repeat(2,1fr)",
                lg: "repeat(3,1fr)",
            }, gap: 2 }, getRootProps(), { children: (_a = props.listData) === null || _a === void 0 ? void 0 : _a.map((item) => (_jsx(ListItemVariables, Object.assign({ variables: item }, { children: _jsx(RadioItem, Object.assign({ item: item, dataValue: props.variableData.value, getRadioProps: getRadioProps, isDisabled: props.isDisabled }, props)) }), item.id))) })));
});
const DropdownListComponent = defineComponent({
    name: "Dropdown List",
    type: "list",
    icon: "SelectInput",
    group: "input",
    display: "block",
    defaultProps: {
        label: "Dropdown List",
        placeholder: "Placeholder here",
        data: {
            inputID: "DropdownList",
            field: { type: "relation", options: {} },
            validation: {
                required: false,
            },
        },
        style: "dropdown",
        itemLabel: "",
        itemImage: "",
        itemValue: "",
        verticalAlignment: false,
        hideLabel: false,
        layout: "vertical_component",
    },
    propDefinition: {
        label: { group: "Design", type: "expression", options: {} },
        placeholder: {
            group: "Design",
            type: "expression",
            options: {},
        },
        data: {
            group: "Design",
            type: "state",
            options: {
                inputID: "DropdownList",
                field: { type: "relation", options: {} },
            },
        },
        style: {
            group: "Design",
            type: "string",
            options: {
                format: "select",
                options: [
                    {
                        label: "Dropdown",
                        value: "dropdown",
                    },
                    {
                        label: "Text Choice",
                        value: "text_choice",
                    },
                    {
                        label: "Image Choice",
                        value: "image_choice",
                    },
                ],
            },
        },
        itemLabel: {
            group: "Design",
            type: "expression",
            options: {},
            section: "dropdown",
        },
        itemImage: {
            group: "Design",
            type: "string",
            options: {
                format: "image",
            },
            section: "dropdown",
            hideOrShownOn: {
                type: "shownOn",
                expression: {
                    type: "formula",
                    formula: `style == "image_choice"`,
                },
            },
        },
        itemValue: {
            group: "Design",
            type: "expression",
            options: {},
            section: "dropdown",
        },
        verticalAlignment: {
            group: "Design",
            type: "boolean",
            options: {},
            section: "dropdown",
            hideOrShownOn: {
                type: "hideOn",
                expression: {
                    type: "formula",
                    formula: `style == "dropdown"`,
                },
            },
        },
        hideLabel: {
            group: "Design",
            type: "boolean",
            options: {},
            section: "dropdown",
            hideOrShownOn: {
                type: "shownOn",
                expression: {
                    type: "formula",
                    formula: `style == "image_choice"`,
                },
            },
        },
        layout: {
            group: "Design",
            section: "dropdown",
            hideOrShownOn: {
                type: "shownOn",
                expression: {
                    type: "formula",
                    formula: `style == "image_choice"`,
                },
            },
            type: "string",
            options: {
                format: "button",
                options: [
                    {
                        icon: "IconTopLeft",
                        value: "vertical_component",
                    },
                    {
                        icon: "IconLeft",
                        value: "horizontal_component",
                    },
                ],
            },
        },
    },
    Editor: () => __awaiter(void 0, void 0, void 0, function* () {
        return {
            data: (props) => {
                const currentPage = RuntimeContext.useSelectState((state) => state.app.maps.page[state.control.page]);
                const tables = EditorContext.useSelectState((state) => state.schema.tables);
                const views = EditorContext.useSelectState((state) => state.schema.views);
                const insights = EditorContext.useSelectState((state) => state.schema.insights);
                const tablesAndViews = React.useMemo(() => tables.concat(views, insights), [tables, views, insights]);
                const pageView = React.useMemo(() => tablesAndViews.find((view) => view.name === currentPage.sourceID), [currentPage.sourceID]);
                return (_jsx(props.PropertyEditor.state, { propDef: props.propDef, propName: props.propName, value: props.value, onChange: props.onChange, view: pageView }));
            },
        };
    }),
    Component: (props) => {
        var _a, _b;
        const boxWidth = React.useRef(null);
        const [selected, setSelected] = React.useState("");
        const [search, setSearch] = React.useState("");
        const sourceHooks = useSourceHooks(props.source);
        const { data: listData = [] } = sourceHooks.useFetchList();
        const title = useEvaluate(props.properties.label);
        const placeholder = useEvaluate(props.properties.placeholder);
        const data = useRuntimeInput(props.properties.data, props.data.page.row, {
            validate: (value) => {
                if (!value) {
                    return { error: "Required field, Please enter the data" };
                }
            },
        });
        const isVerticalAlign = React.useMemo(() => props.properties.verticalAlignment, [props.properties.verticalAlignment]);
        const styleType = React.useMemo(() => props.properties.style, [props.properties.style]);
        // @ts-ignore
        const { isDisabled } = useDisableCondition(props.instance);
        const disabledProperties = {
            border: "1px solid rgb(226, 232, 240)",
            pointerEvents: isDisabled ? "none" : "auto",
            background: isDisabled && "#E7E7E7",
            color: isDisabled && "#A2A2A2",
        };
        return (_jsxs(Box, { children: [_jsx(InputComponentLabel, { title: title, name: "dropdown-list", isRequired: (_a = props.properties.data.validation) === null || _a === void 0 ? void 0 : _a.required, isEmpty: title.length === 0 }), styleType === "dropdown" && (_jsxs(Menu, Object.assign({ autoSelect: false }, { children: [_jsx(MenuButton, Object.assign({ as: Button, style: disabledProperties, bgColor: "surface.900", ref: boxWidth, textAlign: "left", borderRadius: "md", variant: "outline", rightIcon: _jsx(ChevronDown, {}), width: "100%", _focus: { shadow: "0px" }, fontWeight: "normal", "data-test": "dropdown-list-menu-button" }, { children: data.value ? selected : placeholder })), _jsxs(MenuList, Object.assign({ border: "0px", width: (_b = boxWidth === null || boxWidth === void 0 ? void 0 : boxWidth.current) === null || _b === void 0 ? void 0 : _b.offsetWidth, zIndex: 3 }, { children: [_jsx(Box, Object.assign({ paddingX: "0.8rem" }, { children: _jsx(Input, { value: search, onChange: (e) => setSearch(e.target.value), placeholder: "Search...", mb: 2, autoFocus: true }) })), _jsxs(Box, Object.assign({ maxHeight: "11.5rem", overflowY: "auto" }, { children: [_jsx(Text, Object.assign({ color: "gray", py: 2, px: 4, style: { cursor: "default" } }, { children: placeholder })), listData === null || listData === void 0 ? void 0 : listData.map((item) => (_jsx(ListItemVariables, Object.assign({ variables: item }, { children: _jsx(Option, Object.assign({ item: item, onChange: data.onChange, dataValue: data.value, setSelected: setSelected }, props, { "data-test": "dropdown-list-option", search: search })) }), item.id)))] }))] }))] }))), (styleType === "text_choice" || styleType === "image_choice") && (_jsx(CustomRadioGroup, Object.assign({ variableData: data, listData: listData, isVerticalAlign: isVerticalAlign, isDisabled: isDisabled }, props))), !!data.validation.error && (_jsx(Text, Object.assign({ fontSize: "sm", color: "red", "data-test": "text-input-error" }, { children: data.validation.error })))] }));
    },
});
export default DropdownListComponent;
//# sourceMappingURL=dropdownList.js.map